!====================================================================
! file bt_int.f90
!====================================================================
!         EFFECTIVE METHODS FOR SOLVING BANDED TOEPLITZ SYSTEMS
!                     by D.A. Bini and B. Meini
!               (bini@dm.unipi.it    meini@dm.unipi.it)
!           Fortran 90 Program version 1.0, October 30, 1997
!--------------------------------------------------------------------
!                   S C A L A R        C A S E
!====================================================================
!                           INTERFACE FILE
!====================================================================
! This files contains modules and interfaces needed by the
! subroutines of file bt_sub.f90
!==================================================================== 
MODULE comtr
  REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: uv, lzu
END MODULE comtr
MODULE accumulo
  REAL(KIND(0.d0)), DIMENSION(:,:), POINTER   :: trs, tas, bevs, thbs
  REAL(KIND(0.d0)), DIMENSION(:,:,:), POINTER :: dfts, ts
  INTEGER                                     :: ib, iter
END MODULE accumulo
MODULE bt_interface
  INTERFACE 
     SUBROUTINE bt_cr(a,v,ha,r,err,eps)
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: a, v
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: ha,  r
       REAL(KIND(0.d0))                          :: err, eps
     END SUBROUTINE bt_cr
  END INTERFACE
  INTERFACE
     SUBROUTINE bt_bcr(a,v,ha,r,b,bev,dft,tr,ta,thb,t)
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER       :: a, v, b
       REAL(KIND(0.d0)), DIMENSION(:), POINTER         :: ha, r
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER       :: t, dft, bev
       REAL(KIND(0.d0)), DIMENSION(:), POINTER         :: thb, ta, tr
     END SUBROUTINE bt_bcr
  END INTERFACE
  INTERFACE
     SUBROUTINE btl_inv(dft,v,a,t,eps)
       REAL(KIND(0.d0))                          :: eps
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: dft, v, a, t
     END SUBROUTINE btl_inv
  END INTERFACE
  INTERFACE
     SUBROUTINE atax(dft,u,v,tdft,x,y)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: dft, tdft
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: u, v, x, y
     END SUBROUTINE atax
  END INTERFACE
  INTERFACE
     SUBROUTINE aatx(dft,u,v,tdft,x,y)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: dft, tdft
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: u, v, x, y
     END SUBROUTINE aatx
  END INTERFACE
  INTERFACE
     SUBROUTINE ax(dft,u,v,x)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: dft
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: u, v, x
     END SUBROUTINE ax
  END INTERFACE
  INTERFACE
     SUBROUTINE atx(tdft,x)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: tdft
       REAL(KIND(0.D0)), DIMENSION(:), POINTER     :: x
     END SUBROUTINE atx
  END INTERFACE
  INTERFACE
     SUBROUTINE acr(a,ha,r,t,dft,thb,tr,ta)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: a, t, dft
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: ha, thb, tr, ta,r
     END SUBROUTINE acr
  END INTERFACE
  INTERFACE
     SUBROUTINE bcr(b,tb,t,dft,thb,tr,ta)
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: t, dft, b, tb
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: thb, tr, ta
     END SUBROUTINE bcr
  END INTERFACE
  INTERFACE
     SUBROUTINE a0invv(t,v,w,e)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: t
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: v, w
       INTEGER                                   :: e
     END SUBROUTINE a0invv
  END INTERFACE
  INTERFACE
     SUBROUTINE av1(dft,s,w)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: dft
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: s, w
     END SUBROUTINE av1
  END INTERFACE
  INTERFACE
     SUBROUTINE av2(dft,thb,tr,ta,s,w)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: dft
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: thb, tr, ta, s, w
     END SUBROUTINE av2
  END INTERFACE
  INTERFACE 
     SUBROUTINE luv(a,b,s,z)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: a, b, s, z
     END SUBROUTINE luv
  END INTERFACE
  INTERFACE
     SUBROUTINE luvs(a,b,s,z)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: a, b, s, z
     END SUBROUTINE luvs
  END INTERFACE
  INTERFACE
     SUBROUTINE cgata(dft,u,v,tdft,x,x0,eps)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: dft, tdft
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: u, v
       REAL(KIND(0.d0)), DIMENSION(:), POINTER     :: x, x0
       REAL(KIND(0.d0))                              :: eps
     END SUBROUTINE cgata
  END INTERFACE
  INTERFACE
     SUBROUTINE cgaat(dft,u,v,tdft,x,x0,eps)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: dft, tdft
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: u, v
       REAL(KIND(0.d0)), DIMENSION(:), POINTER     :: x, x0
       REAL(KIND(0.d0))                              :: eps
     END SUBROUTINE cgaat
  END INTERFACE
  INTERFACE
     SUBROUTINE test_stoc(dft,thb,tr,ta,err)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: dft
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: thb, tr,ta
       REAL(KIND(0.d0)) :: err
     END SUBROUTINE test_stoc
  END INTERFACE
  INTERFACE
     SUBROUTINE vcr(v,t,dft,thb,tr,ta)
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: thb, tr, ta
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER       :: v,t, dft
     END SUBROUTINE vcr
  END INTERFACE
  INTERFACE
     SUBROUTINE test(a,v,r,err)
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: a, v
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: r
       REAL(KIND(0.d0)) :: err
     END SUBROUTINE test
  END INTERFACE
  INTERFACE
     SUBROUTINE solve1(ha,bm0,r1,tr,thb,dft,t,iter,b,x,eps)
       REAL(KIND(0.d0))                                  :: eps
       REAL(KIND(0.d0)), DIMENSION(:), POINTER           :: ha, bm0,&
            & tr, thb 
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER         :: dft, t, b&
            & , x
       REAL(KIND(0.d0))                                  :: r1
       INTEGER                                           :: iter
     END SUBROUTINE solve1
  END INTERFACE
  INTERFACE
     SUBROUTINE solve2(ta,tr,thb,dft,t,tb,iter,x)
       IMPLICIT NONE
       REAL(KIND(0.d0)), DIMENSION(:), POINTER           :: ta, tr,&
            & thb
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER         :: dft,t, tb&
            & , x
       INTEGER                                           :: iter
     END SUBROUTINE solve2
  END INTERFACE
  INTERFACE
     SUBROUTINE recover(conv,bm0,ha,r1,dft,tr,ta,thb,t,b,x,eps)
       REAL(KIND(0.d0))                                  :: r1, eps
       REAL(KIND(0.d0)), DIMENSION(:), POINTER           :: bm0, ha,&
            &  tr, ta, thb 
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER         :: b, x, dft&
            & , t
       LOGICAL  :: conv
     END SUBROUTINE recover
  END INTERFACE
  INTERFACE
     SUBROUTINE store(dft,tr,ta,thb,tb,t)
       REAL(KIND(0.d0)), DIMENSION(:), POINTER           :: tr, ta, thb
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER         :: dft, tb, t
     END SUBROUTINE store
  END INTERFACE
  INTERFACE
     SUBROUTINE solve_cr(a,b,x,eps,epsgc)
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER       :: a, x, b
       REAL(KIND(0.d0))                                :: eps, epsgc
     END SUBROUTINE solve_cr
  END INTERFACE
  INTERFACE
     SUBROUTINE fft_cr(a,v,r,dft,tr,ta,thb,t,eps)
       REAL(KIND(0.d0))                                :: eps
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER       :: a, v
       REAL(KIND(0.d0)), DIMENSION(:), POINTER         :: r
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER       :: t, dft
       REAL(KIND(0.d0)), DIMENSION(:), POINTER         :: thb, ta, tr
     END SUBROUTINE fft_cr
  END INTERFACE
  INTERFACE
     SUBROUTINE ha_solve(ta,tv,tb,tdft,x,eps)
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: tdft
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: ta, tv, tb
       REAL(KIND(0.d0)), DIMENSION(:), POINTER     :: x
       REAL(KIND(0.d0))                              :: eps
     END SUBROUTINE ha_solve
  END INTERFACE
  INTERFACE
     SUBROUTINE hathax(ta,tv,tb,tdft,x,y)
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: tdft
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: ta, tv, tb
       REAL(KIND(0.d0)), DIMENSION(:), POINTER     :: x,y
     END SUBROUTINE hathax
  END INTERFACE
  INTERFACE
     SUBROUTINE hax(ta,tv,tb,x)
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: ta, tv, tb
       REAL(KIND(0.d0)), DIMENSION(:), POINTER     :: x
     END SUBROUTINE hax
  END INTERFACE
  INTERFACE
     SUBROUTINE hatx(tdft,x)
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: tdft
       REAL(KIND(0.d0)), DIMENSION(:), POINTER     :: x
     END SUBROUTINE hatx
  END INTERFACE
  INTERFACE
     SUBROUTINE solvel(ha,bm0,ta,thb,dft,b,xtot,eps)
       REAL(KIND(0.d0)), DIMENSION(:), POINTER   :: ha, bm0, ta, thb
       REAL(KIND(0.d0))                              :: eps
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: b, xtot, dft
     END SUBROUTINE solvel
  END INTERFACE
  INTERFACE
     SUBROUTINE  hathalx(tha,that,x,y)
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER :: tha,that
       REAL(KIND(0.d0)), DIMENSION(:), POINTER     :: x,y
     END SUBROUTINE hathalx
  END INTERFACE
  INTERFACE
     SUBROUTINE halx(tha,x)
       REAL(KIND(0.d0)), DIMENSION(:,:), POINTER   :: tha
       REAL(KIND(0.d0)), DIMENSION(:), POINTER     :: x
     END SUBROUTINE halx
  END INTERFACE
END MODULE bt_interface
